<?
/***************************************************************************
 *   Copyright (C) 2005 by J Randolph Smith                                *
 *   snortsms@servangle.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
include_once("func_Log.php");





/***SNORT Rule Disection Class***/

class Rule
	{
	var $raw;
	var $rulearray;
	var $content_cnt;
	var $uricontent_cnt;
	var $option_cnt;
	var $ref_cnt;
	var $direction;
	var $error;
	var $hash_header;

	function Rule ()
		{
		$this->rulearray = array();
		//$rulearray['header'] = array();
		//$rulearray['options'] = array();
		$this->content_cnt = -1;
		$this->uricontent_cnt = -1;
		$this->option_cnt = -1;
		$this->ref_cnt = -1;	
		$this->direction = array("->","<>");
		$this->error = array();
		/* Define the header lookup table */
		$this->hash_header['action'] = 0;
		$this->hash_header['protocol'] = 1;
		$this->hash_header['src_ip'] = 2;
		$this->hash_header['src_port'] = 3;
		$this->hash_header['flow'] = 4;
		$this->hash_header['dst_ip'] = 5;
		$this->hash_header['dst_port'] = 6;
		}
		
	function Set ($varname,$value) 		{ $this->$varname=$value; }
	function Get ($varname) 		{ return ($this->$varname); }

	function Parse($rule)
		{
		$this->raw = $rule;

		/*** Get the Header and Option***/
		//preg_match("/(.*)\((.*)\)/", $rule, $matches);
		if ( ! preg_match("/([^\(]+)\((.*)\)/", $rule, $matches) )
			{
			array_push($this->error,"Rule Header or Options parsing failed");
			LogEvent("Error: " . $this->error);
			return false;
			}
		$rule_header = $matches[1];
		$rule_options = $matches[2];
		/*** Get all the Header values ****/	
		if ( ! preg_match_all("/([^\s]+)\s+?/", $rule_header, $header) )
			{
			array_push($this->error,"Rule Header parsing failed");
			LogEvent("Error: " . $this->error);
			return false;
			}
		$this->rulearray['header'] = $header[0];
		/*** Get all the Option tokens ****/	
		//preg_match_all("/(\w+):?([^;]+)?;\s?/", $rule_options, $option);
		if ( ! preg_match_all("/(\w+):?(\"[^\"]+\"|[^;]+)?;\s?/", $rule_options, $option) )
			{
			array_push($this->error,"Rule Options parsing failed");
			LogEvent("Error: " . $this->error);
			return false;
			}
		$count = count($option[1]);
		
/*
print"<PRE>\n";
print_r($option);
print"</PRE>\n";
*/


		for($i=0;$i<$count;$i++)
			{
			switch ($option[1][$i])
				{
				case "content":
				$this->content_cnt++;
				$this->rulearray['content'][$this->content_cnt]['content'] = $option[2][$i];
				break;
				
				case "uricontent":
				$this->content_cnt++;
				$this->rulearray['content'][$this->content_cnt]['uricontent'] = $option[2][$i];
				break;
				
				case "depth":
				$this->rulearray['content'][$this->content_cnt]['depth'] = $option[2][$i];
				break;
				
				case "offset":
				$this->rulearray['content'][$this->content_cnt]['offset'] = $option[2][$i];
				break;
				
				case "distance":
				$this->rulearray['content'][$this->content_cnt]['distance'] = $option[2][$i];
				break;
				
				case "within":
				$this->rulearray['content'][$this->content_cnt]['within'] = $option[2][$i];
				break;			
				
				case "isdataat":
				$this->rulearray['content'][$this->content_cnt]['isdataat'] = $option[2][$i];
				break;
				
				case "nocase":
				$this->rulearray['content'][$this->content_cnt]['nocase'] = true;
				break;
				
				case "rawbytes":
				$this->rulearray['content'][$this->content_cnt]['rawbytes'] = true;
				break;
				
				case "reference":
				$this->ref_cnt++;
				$this->rulearray['reference'][$this->ref_cnt] = $option[2][$i];
				break;
								
				default:
				$this->option_cnt++;
				$this->rulearray['options'][$this->option_cnt]['key'] = strtolower($option[1][$i]);
				$this->rulearray['options'][$this->option_cnt]['val'] = $option[2][$i];
				break;
				}
			
			}

		/*** Was rule disection a success ***/
		return ($this->_Validate());
		}


	/*********** Returns the rule as a plain text string 	**********/
	function GenRule()
		{	
		$gen_rule = "";
		$count = count($this->rulearray['header']);
		for($i=0;$i<$count;$i++)
			{	
			$gen_rule .= $this->rulearray['header'][$i];			
			}
		$gen_rule .= "(";
			
		/*** Loop content ***/
		if( isset($this->rulearray['content']) )
			{
			$count = count($this->rulearray['content']);
			for($i=0;$i<$count;$i++)
				{
				if ( isset($this->rulearray['content'][$i]['content']) )
					{
					$gen_rule .= "content:".$this->rulearray['content'][$i]['content'] . "; ";
					}
				if ( isset($this->rulearray['content'][$i]['uricontent']) )
					{
					$gen_rule .= "uricontent:".$this->rulearray['content'][$i]['uricontent'] . "; ";
					}
				if ( isset($this->rulearray['content'][$i]['offset']) )
					{
					$gen_rule .= "offset:".$this->rulearray['content'][$i]['offset'] . "; ";
					}
				if ( isset($this->rulearray['content'][$i]['isdataat']) )
					{
					$gen_rule .= "isdataat:".$this->rulearray['content'][$i]['isdataat'] . "; ";
					}
				if ( isset($this->rulearray['content'][$i]['depth']) )
					{
					$gen_rule .= "depth:".$this->rulearray['content'][$i]['depth'] . "; ";
					}
				if ( isset($this->rulearray['content'][$i]['within']) )
					{
					$gen_rule .= "within:".$this->rulearray['content'][$i]['within'] . "; ";
					}
				if ( isset($this->rulearray['content'][$i]['distance']) )
					{
					$gen_rule .= "distance:".$this->rulearray['content'][$i]['distance'] . "; ";
					}
				if ( isset($this->rulearray['content'][$i]['nocase']) )
					{
					$gen_rule .= "nocase; ";
					}
				if ( isset($this->rulearray['content'][$i]['rawbytes']) )
					{
					$gen_rule .= "rawbytes; ";
					}					
				}
			}
			
		/*** Loop options ***/	
		$count = count($this->rulearray['options']);
		for($i=0;$i<$count;$i++)
			{	
			if ($this->rulearray['options'][$i]['val'] != "")
				{
				$gen_rule .= $this->rulearray['options'][$i]['key'] . ":" . $this->rulearray['options'][$i]['val'] . "; ";
				}
			else	
				{
				$gen_rule .= $this->rulearray['options'][$i]['key'] . "; ";
				}
			}
			
		/*** Loop references ***/
		if( isset($this->rulearray['reference']) )
			{
			$count = count($this->rulearray['reference']);
			for($i=0;$i<$count;$i++)
				{
				$gen_rule .= "reference:".$this->rulearray['reference'][$i] . "; ";			
				}
			}
			
		$gen_rule .= ")";
				
		return ($gen_rule);
		}


	function GetHeaderItem($section)
		{
		/* Translate the cononical header name to numeric index value */		
		if(isset($this->hash_header[$section]))
			{
			$index = $this->hash_header[$section];
			$return = $this->rulearray['header'][$index];
			}
		else
			{
			$return = "ERROR: Unknown header field passed to member function";
			}
		
		return($return);
		}
		
		
	function GetOption($option)
		{
		$count = count($this->rulearray['options']);
		for($i=0;$i<$count;$i++)
			{
			if ($this->rulearray['options'][$i]['key'] == $option)
				return($this->rulearray['options'][$i]['val']);			
			}		
		return NULL;
		}

		
		
		
	function GetReferences()
		{
		$return = "";
		if(isset($this->rulearray['reference']))
			{
			$return = $this->rulearray['reference'];
			}
			
		return($return);
		}


	function SetOption($option,$value)
		{
		$count = count($this->rulearray['options']);
		for($i=0;$i<$count;$i++)
			{
			if ($this->rulearray['options'][$i]['key'] == $option)
				{
				// *** Option found, so update the value.
				$this->rulearray['options'][$i]['val'] = $value;	
				return;
				}

			}
		// *** Option not found, so add it.
		$this->rulearray['options'][$count]['key'] = $option;
		$this->rulearray['options'][$count]['val'] = $value;
		return;
		}

		
		
	function _Validate()
		{
		$header_count = count($this->rulearray['header']);
		$options = (is_array($this->rulearray['options']) ? true : false);
		$msg = $this->GetOption("msg");
		if ($header_count <> 7)
			{
			array_push($this->error,"Rule Header Error");
			}
		if (!$options)
			{
			array_push($this->error,"Rule Options Error");
			}
		if ($msg == "")
			{
			array_push($this->error,"Missing or invalid 'msg:' tag");
			}
			
		if( isset($this->rulearray['content']) )
			{	
			if( is_array($this->rulearray['content']) )
				{
				$key = key($this->rulearray['content']);
				if( $key <= -1 )
					{
					array_push($this->error,"Stray 'content' modifier");
					}
				}
			}
		
		/* Were there any issues ? */
		if(count($this->error) > 0)
			{
			LogEvent("Error: rule validation: " . $this->error);
			return false;
			}
		else
			{
			return true;
			}
		}

		
		
	function ShowTree()
		{
		?>
		<p>
		<pre><b>*** DEBUG ***</b>	
		<?
		print_r($this->rulearray);
		?>
		<b>*************</b>
		</pre>
		</p>
		<?
		}



	/*********** Returns all the 'content' related option as a text string 	**********/
	function GetContentLines()
		{	
		$gen_string = "";
		$break = "";
		$bull = "  ";

		/*** Loop content ***/
		if( isset($this->rulearray['content']) )
			{
			$count = count($this->rulearray['content']);
			for($i=0;$i<$count;$i++)
				{
				if ( isset($this->rulearray['content'][$i]['content']) )
					{
					$gen_string .= $break."content:".$this->rulearray['content'][$i]['content'] . $bull;
					$break = "\n";
					}
				if ( isset($this->rulearray['content'][$i]['uricontent']) )
					{
					$gen_string .= $break."uricontent:".$this->rulearray['content'][$i]['uricontent'] . $bull;
					$break = "\n";
					}
				if ( isset($this->rulearray['content'][$i]['offset']) )
					{
					$gen_string .= "offset:".$this->rulearray['content'][$i]['offset'] . $bull;
					}
				if ( isset($this->rulearray['content'][$i]['isdataat']) )
					{
					$gen_string .= "isdataat:".$this->rulearray['content'][$i]['isdataat'] . $bull;
					}
				if ( isset($this->rulearray['content'][$i]['depth']) )
					{
					$gen_string .= "depth:".$this->rulearray['content'][$i]['depth'] . $bull;
					}
				if ( isset($this->rulearray['content'][$i]['within']) )
					{
					$gen_string .= "within:".$this->rulearray['content'][$i]['within'] . $bull;
					}
				if ( isset($this->rulearray['content'][$i]['distance']) )
					{
					$gen_string .= "distance:".$this->rulearray['content'][$i]['distance'] . $bull;
					}
				if ( isset($this->rulearray['content'][$i]['nocase']) )
					{
					$gen_string .= "nocase". $bull;
					}
				if ( isset($this->rulearray['content'][$i]['rawbytes']) )
					{
					$gen_string .= "rawbytes". $bull;
					}					
				}
			}
				
		return ($gen_string);
		}
		

	/*********** Returns all the 'options' related option as a text string 	**********/
	function GetOptionLines()
		{	
		$gen_string = "";
		$break = "";
		$bull = "   ";
		/*** Loop options ***/	
		$count = count($this->rulearray['options']);
		for($i=0;$i<$count;$i++)
			{	
			if ($this->rulearray['options'][$i]['val'] != "")
				{
				$gen_string .= $break.$this->rulearray['options'][$i]['key'] . ":" . $this->rulearray['options'][$i]['val'] . $bull;
				$break = "\n";
				}
			else	
				{
				$gen_string .= $break.$this->rulearray['options'][$i]['key']  . $bull;
				$break = "\n";
				}
			}
		
		return ($gen_string);
		}
		

		
	/*********** Returns all the 'options' related option as a text string 	**********/
	function GetReferenceLines()
		{	
		$gen_string = "";
		$break = "";
		$bull = "   ";

		/*** Loop references ***/
		if( isset($this->rulearray['reference']) )
			{
			$count = count($this->rulearray['reference']);
			for($i=0;$i<$count;$i++)
				{
				$gen_string .= $break.$this->rulearray['reference'][$i] . $bull;
				$break = "\n";
				}
			}
		
		
		return ($gen_string);
		}
		
		
		
		
		
		
		
		
	}	// *** END: Class
	
	?>
